clear
clc
%All quantities in atomic units
%Set up a uniformly spaced grid, center-of-mass is the origin
step_size=0.001 %In bohr
z=-1.5:step_size:5.0;
Npoints=length(z)
%
format long
%Calculate the reduced mass in atomic units
amu_to_me = 1822.8885300626 %Unit conversion from 1 amu to electron masses
massA = 1.00784 
massB = 2.01410 
reduced_mass = amu_to_me*massA*massB/(massA + massB)
%First, the kinetic energy part
H = zeros(Npoints,Npoints);
H(1,1:2)= [1 -1]; 
H(Npoints,(Npoints - 1):Npoints) = [-1 1];
for i=2:(Npoints - 1)
    H(i,i) = 2;
    H(i,(i-1)) = -1;
    H(i,(i+1)) = -1;
end
H = H/(2*reduced_mass*step_size^2);
% Manz stretch potential parameters
k_Manz_stretch = 0.399 %units are hartree/bohr^2
gamma = 1.170 % units are bohr^-1
for i=1:Npoints
    H(i,i) = H(i,i) + (3*k_Manz_stretch/(5*gamma^2))*(1 - 2.5*exp(-gamma*z(i)) + 1.5*exp(-(5/3)*gamma*z(i)));
end
% Calculate the first ten frequencies
hartree_to_wavenumber = 219474.63
freqs=eig(H)*hartree_to_wavenumber;
'The zero-point energy is (in wavenumber, cm^-1)'
freqs(1)
'The spacing between adjacent vibrational levels is (in wavenumber, cm^-1):'
for i=1:12
    freq_diffs(i) = freqs(i+1) - freqs(i);
end
display(freq_diffs)



