clear
clc
format long
% conversion factors
bohrperangstrom = 1.88973
eV_to_Jpermol = 96491.5666
speed_of_light = 29979245800; % in cm per second
angstroms_per_meter = 10^10;
% step size
stepsize = 0.0001
% optimized geometry (in angstroms)
opt_geom = [
0.073584     0.000000     1.464944
0.067537     0.000000     0.039082
1.306792     0.000000    -0.666160
];
% mass of each atom 
% using most common isotope of each chemical element
masses = [15.9949, 31.9721, 15.9949] % in amu
% convert the atomic masses to kg/mol
masses = masses/1000;
% forcefield parameters (in eV and bohrs and radians)
kstretch = 20.8625;
kbend = 11.5635;
% convert to J/mol and angstroms
kstretch = kstretch*eV_to_Jpermol*bohrperangstrom^2
kbend = kbend*eV_to_Jpermol
d_eq = 1.425875;
theta_eq = 2.083933;
% build the mass-weighted Hessian matrix
MW_Hessian = zeros(9,9);
temp=0
for atom1 = 1:3
    for dir1 = 1:3
        index1 = 3*(atom1-1) + dir1;
        for atom2 = 1:3
            for dir2 = 1:3
                index2 = 3*(atom2-1) + dir2;
                % second derivatives are computed using the central finite
                % difference approximation
                if index1 == index2
                    displaced_geom1 = opt_geom;
                    displaced_geom1(atom1,dir1) = displaced_geom1(atom1,dir1) + stepsize;
                    displaced_geom2 = opt_geom;
                    displaced_geom2(atom2,dir2) = displaced_geom2(atom2,dir2) - stepsize;
                    displaced_potential_1 = calc_potential(displaced_geom1,kstretch,kbend,d_eq,theta_eq);
                    displaced_potential_2 = calc_potential(displaced_geom2,kstretch,kbend,d_eq,theta_eq);
                    opt_potential = calc_potential(opt_geom,kstretch,kbend,d_eq,theta_eq); 
                    temp = (displaced_potential_1 + displaced_potential_2 - 2*opt_potential)/(masses(atom1)*stepsize^2);
                    MW_Hessian(index1,index2) = temp;
                else
                    displaced_geom1 = opt_geom;
                    displaced_geom1(atom1,dir1) = displaced_geom1(atom1,dir1) + stepsize;
                    displaced_geom1(atom2,dir2) = displaced_geom1(atom2,dir2) + stepsize;
                    displaced_geom2 = opt_geom;
                    displaced_geom2(atom1,dir1) = displaced_geom2(atom1,dir1) + stepsize;
                    displaced_geom2(atom2,dir2) = displaced_geom2(atom2,dir2) - stepsize;
                    displaced_geom3 = opt_geom;
                    displaced_geom3(atom1,dir1) = displaced_geom3(atom1,dir1) - stepsize;
                    displaced_geom3(atom2,dir2) = displaced_geom3(atom2,dir2) + stepsize;
                    displaced_geom4 = opt_geom;
                    displaced_geom4(atom1,dir1) = displaced_geom4(atom1,dir1) - stepsize;
                    displaced_geom4(atom2,dir2) = displaced_geom4(atom2,dir2) - stepsize;
                    displaced_potential_1 = calc_potential(displaced_geom1,kstretch,kbend,d_eq,theta_eq);
                    displaced_potential_2 = calc_potential(displaced_geom2,kstretch,kbend,d_eq,theta_eq);
                    displaced_potential_3 = calc_potential(displaced_geom3,kstretch,kbend,d_eq,theta_eq);
                    displaced_potential_4 = calc_potential(displaced_geom4,kstretch,kbend,d_eq,theta_eq);
                    temp = (displaced_potential_1 - displaced_potential_2 - displaced_potential_3 + displaced_potential_4);
                    temp = temp/(4*sqrt(masses(atom1)*masses(atom2))*stepsize^2);
                    MW_Hessian(index1,index2) = temp;
                end
            end
        end
    end
end
% diagonalize the mass-weighted Hessian matrix 
display(MW_Hessian)
eigenvals = real(eig(MW_Hessian))
freqs=sqrt(max(eigenvals,0));
% convert the frequencies to wavenumbers (cm^-1)
freqs=freqs*angstroms_per_meter/(speed_of_light*(2*pi));
display('The frequencies (in wavenumber, cm^-1) calculated from the forcefield are:')
ordered_freqs=sort(round(freqs),'descend')
% experimental freqs from 
% 2016-2017 Handbook of Chemistry and Physics page 105 of section 9
experimental_freqs = [1362 1151 518]
for i = 1:length(experimental_freqs)
    percent_error(i) = round(100*ordered_freqs(i)/experimental_freqs(i) - 100);
end
display('The relative error of the calculated freqs is:')
display(percent_error)





